
#import sys
#sys.path.append("./lib")

#from LedRing import LedRing

from mymachine import ledRing




BLACK = (0, 0, 0)    # Black color
RED   = (255, 0, 0)  # Red color
GREEN = (0, 255, 0)  # Green color
BLUE  = (0, 0, 255)  # Blue color
WHITE  = (255, 255, 255)  # White color


BACKGROUND  = BLACK
MARKER_3HOUR = (0, 0, 60)
MARKER_HOUR  = (0, 0, 20)
HAND_HOUR    = RED
HAND_MINUTE  = GREEN
HAND_SECONDS = (100, 100, 100)

BRIGHTNESS = 0.05


try:
    import platform
    system = platform.system()
    if (system == "Windows"):
        BACKGROUND  = BLACK
        MARKER_3HOUR = (0, 0, 255)
        MARKER_HOUR  = (0, 0, 127)
        HAND_HOUR    = RED
        HAND_MINUTE  = GREEN
        HAND_SECONDS = WHITE

        BRIGHTNESS = 1
except:
    pass


    

class LedRingClockDisplay:


    def __init__(self):
        self.brightnessSecondsHand = 1.0
        
        self.ledRing = ledRing
        self.numLeds = ledRing.n
        self.__clear()
        self.__drawDial()
        self.__write()

        return



    def __clear(self):
        self.ledRing.fill(BACKGROUND)
        return



    def __write(self):
        
        for ledId in range(self.numLeds):
           self.ledRing[ledId] = self.__calcBrightness(self.ledRing[ledId], BRIGHTNESS)

        self.ledRing.write()
        return




    def __drawDial(self):
        # Zifferblatt

        for ledId in range(self.numLeds):
            # Iterate through each LED in the ring
            if (ledId % 2 == 0):
               self.ledRing[ledId] = MARKER_HOUR   # Set the color of the corresponding LED
            if (ledId % 6 == 0):
                self.ledRing[ledId] = MARKER_3HOUR  # Set the color of the corresponding LED

        return
        


    
    

    def __calcHourHandPosition(self, hour, minute):
        position = (float(hour % 12) * 60 + minute) / (12 * 60)
        return position

    def __calcMinuteHandPosition(self, minute, seconds):
        position = (float(minute) * 60 + seconds) / (60 * 60)
        return position

    def __calcSecondHandPosition(self, seconds, millisecs):
        position = (float(seconds) * 1000 + millisecs) / (60 * 1000)
        return position

    def __calcLedIdAndDeviation(self, position):
        ledId = int(position * self.numLeds)
        deviation = position * self.numLeds - int(position * self.numLeds)
        return ledId, deviation




    
    def __drawHourHand(self, hour, minute):
        # Stundenzeiger
        position = self.__calcHourHandPosition(hour, minute)
        ledId, deviation = self.__calcLedIdAndDeviation(position)
        color = HAND_HOUR
        self.ledRing[ledId] = color  # Set the color of the corresponding LED
        return



    def __drawMinuteHand(self, hour, minute, seconds, millisecs):
        # Minutenzeiger
        position = self.__calcMinuteHandPosition(minute, seconds)
        ledId, deviation = self.__calcLedIdAndDeviation(position)
        hourPosition = self.__calcHourHandPosition(hour, minute)
        hourLedId, hourDeviation = self.__calcLedIdAndDeviation(hourPosition)
        color = HAND_MINUTE
        if ledId == hourLedId:
            if millisecs < 500:
                return
        self.ledRing[ledId] = color  # Set the color of the corresponding LED
        return

    
    
    def __drawSecondsHand(self, seconds, millisecs):
        # Sekundenzeiger
        position = self.__calcSecondHandPosition(seconds, millisecs)
        ledId, deviation = self.__calcLedIdAndDeviation(position)

        
        prevLedId = (ledId - 1 + self.numLeds) % self.numLeds        
        nextLedId = (ledId + 1) % self.numLeds
        
        id = prevLedId
        brightness = self.__fadeOut(deviation)
        color = self.__calcBrightness(HAND_SECONDS, brightness)
#        self.ledRing[id] = self.__calcMixColorAdditive(color, self.ledRing[id])
 

        id = ledId
        brightness = self.__fade(deviation)
        color = self.__calcBrightness(HAND_SECONDS, brightness)
        self.ledRing[id] = self.__calcMixColorAdditive(color, self.ledRing[id])


        id = nextLedId
        brightness = self.__fadeIn(1 - deviation)
        color = self.__calcBrightness(HAND_SECONDS, brightness)
#        self.ledRing[id] = self.__calcMixColorAdditive(color, self.ledRing[id])

        return

    
    
#    def __fading(self, deviation):
#        # Sekundenzeiger
#        
#        if deviation < 0.0:
#            return 1.0 ## abs(deviation)
#        
#        brightness = (1.0 - abs(deviation)) * 1.2
#        brightness = min(brightness, 1.0)
#        return brightness

    
    
    def __fadeOut(self, deviation):
        # Sekundenzeiger

        brightness = (1.0 - abs(deviation))
        brightness = min(brightness, 1.0)
        return brightness

    def __fadeIn(self, deviation):
        # Sekundenzeiger
        
        brightness = (1.0 - abs(deviation))
        brightness = min(brightness, 1.0)
        return brightness


    def __fade(self, deviation):
        # Sekundenzeiger
        
        if deviation < .2:
            return .2
        if deviation < .4:
            return .4
        if deviation < .6:
            return .6
        if deviation < .8:
            return .8
        return 1.0

        if deviation < .2:
            return .3
        if deviation < .4:
            return .6
        if deviation < .6:
            return 1.0
        if deviation < .8:
            return .6
        return .3
        
#        brightness = int((abs(deviation - 0.5) * 10)) * 3.0
#        brightness = 0
#        if deviation > 0.0:
#        brightness = max(brightness, 0.0)
#        brightness = 1.0 ## abs(deviation)
#        if (int(deviation * 10) % 2) != 0:
#           brightness = 0.5
           
#        return brightness

    
    
    def __oldfade(self, deviation):
        # Sekundenzeiger
        
        brightness = 1.0 - 2 * (abs(deviation - 0.5))
#        brightness = 0
#        if deviation > 0.0:
        brightness = max(brightness, 0.0)
#        brightness = 1.0 ## abs(deviation)
#        if (int(deviation * 10) % 2) != 0:
#           brightness = 0.5
           
        return brightness

    
    
    def displayTime(self, hour, minute, seconds, millisecs):
        self.__clear()
        self.__drawDial()
        self.__drawHourHand(hour, minute)
        self.__drawMinuteHand(hour, minute, seconds, millisecs)
        self.__drawSecondsHand(seconds, millisecs)
        
        self.__write()
#        print("NeoPixel[" + str(0) + "] = " + str(self.neoRing[0]))

        return
        



    def __calcBrightness(self, color, brightness):
        r, g, b = color
        r = int(r * brightness)
        g = int(g * brightness)
        b = int(b * brightness)
        return (r, g, b)




    def __calcMixColorAdditive(self, color1, color2):
        r = max(color1[0], color2[0])
        g = max(color1[1], color2[1])
        b = max(color1[2], color2[2])
        return (r, g, b)

