# Bibliotheken laden


try:
    from machine import Pin
except:
    import keyboard

    LEFTBUTTON_PIN   = 0
    RIGHTBUTTON_PIN  = 15

    class Pin():
        IN = 0
        PULL_UP = 0
        PULL_DOWN = 1
        
        def __init__(self, pin, io, pull):
            self.pin = pin
            self.pull = pull
            self.pressedLevel = pull    # tricky
            return
        def value(self):
            if self.pin == LEFTBUTTON_PIN:
                if keyboard.is_pressed("left"):
                    return self.pressedLevel
            
            if self.pin == RIGHTBUTTON_PIN:
                if keyboard.is_pressed("right"):
                    return self.pressedLevel
                    
            return abs(self.pressedLevel - 1)        
    # dummy class to enable testing under Windows






def msTicks():
    try:
        from time import ticks_ms
        return ticks_ms()
    except:
        from time import time
        return time() * 1000








class Debouncer:

    def __init__(self, bounceTimeMs=10):

        self.bounceTimeMs = bounceTimeMs   # store bounce time setting (Prellzeit)

        self.lastValue = False
        self.valueChangedTime = 0

        return



    def debounce(self, value):

        timestamp = msTicks()
        
        #check if in debounce interval, if so ignore current value
        if (timestamp < self.valueChangedTime + self.bounceTimeMs):
            return self.lastValue

        if value != self.lastValue:
            self.valueChangedTime = timestamp
        self.lastValue = value

        return value












#from enum import Enum
#class State(Enum):
class State():
    NOT_PRESSED = 1
    PRESSED_IN_SHORT_INTERVAL = 2
    PRESSED_IN_LONG_INTERVAL = 3
    PRESSED = 4


 



class Button():

    # values of pressedLevel
    LOW = 0
    HIGH = 1
    
    # values or returned events
    class Event():
        NONE = 0
        PRESSED = 1
        RELEASED = 2
        LONG_PRESS_DETECTED = 3
        LONG_PRESS_RELEASED = 4


    
    def __init__(self, pin, pressedLevel, bounceTimeMs=10):
    
        self.pressedLevel = pressedLevel
        if (self.pressedLevel == 0):
            self.btn = Pin(pin, Pin.IN, Pin.PULL_UP)
        else:
            self.btn = Pin(pin, Pin.IN, Pin.PULL_DOWN)

        self.debouncer = Debouncer(bounceTimeMs=bounceTimeMs)

        self.longTimeMs = 0
        self.pressedStartTime = 0
        self.state = State.NOT_PRESSED

        return





    def setLongTimeMs(self, longTimeMs):
    
        self.longTimeMs = longTimeMs  # unterscheidet kurz/lang gedrückt
        return






    def event(self):

        timestamp = msTicks()
        pressedValue = self.debouncer.debounce(self.__isPressed())
        
        if (self.state == State.NOT_PRESSED):
            if (pressedValue == True):
                # Knopf wurde gedrückt
                self.pressedStartTime = timestamp
                if self.longTimeMs == 0:
                    self.state = State.PRESSED
                else:
                    self.state = State.PRESSED_IN_SHORT_INTERVAL
                return Button.Event.PRESSED
                
        if (self.state == State.PRESSED_IN_SHORT_INTERVAL):
            if (pressedValue == False):
                self.state = State.NOT_PRESSED
                # short press event detected 
                return Button.Event.RELEASED
            if (timestamp >= (self.pressedStartTime + self.longTimeMs)):
                # long press event detected 
                self.state = State.PRESSED_IN_LONG_INTERVAL
                return Button.Event.LONG_PRESS_DETECTED

        if (self.state == State.PRESSED_IN_LONG_INTERVAL):
            if (pressedValue == False):
                # long press event detected in one of the previous calls and already returned
                self.state = State.NOT_PRESSED
                return Button.Event.LONG_PRESS_RELEASED

        if (self.state == State.PRESSED):
            if (pressedValue == False):
                self.state = State.NOT_PRESSED
                return Button.Event.RELEASED

        return Button.Event.NONE




    def status(self):

        return self.state


    def __isPressed(self):

        return (self.pressedLevel == self.btn.value())





if __name__ == "__main__":

    from time import sleep
    

    rightButton = Button(15, Button.LOW, bounceTimeMs=10)
    rightButton.setLongTimeMs(3000)

    leftButton = Button(0, Button.LOW, bounceTimeMs=10)

    while True:
        event = rightButton.event()
        
        if event == Button.Event.PRESSED:
            print (">>>>>>>> rightButton: PRESSED")
        if event == Button.Event.LONG_PRESS_DETECTED:
            print (">>>>>>>> rightButton: LONG_PRESS_DETECTED")
        if event == Button.Event.LONG_PRESS_RELEASED:
            print (">>>>>>>> rightButton: LONG_PRESS_RELEASED")
        if event == Button.Event.RELEASED:
            print (">>>>>>>> rightButton: RELEASED")

        event = leftButton.event()
        
        if event == Button.Event.PRESSED:
            print (">>>>>>>> leftButton: PRESSED")
        if event == Button.Event.LONG_PRESS_DETECTED:
            print (">>>>>>>> leftButton: LONG_PRESS_DETECTED")
        if event == Button.Event.LONG_PRESS_RELEASED:
            print (">>>>>>>> leftButton: LONG_PRESS_RELEASED")
        if event == Button.Event.RELEASED:
            print (">>>>>>>> leftButton: RELEASED")

        sleep(0.01)
   




