
try:
    from neopixel import NeoPixel
    from machine import Pin
except:
    from neopixelsim import NeoPixel
    def Pin(x):
        return x




   

class LedRing:


    def __init__(self, pin, n, offset=0):
        self.neoRing = NeoPixel(Pin(pin), n)
        self.n = n
        self.indexOffset = offset

        return



    def clear(self):
        self.neoRing.fill((0,0,0))
        return



    def fill(self, color, n=0, startId=0):
        if n == 0:
            self.neoRing.fill(color)
            return
        
        ledId = startId
        for i in range(n):
            self.neoRing[ledId] = color
            ledId = self.nextLedId(ledId)

        return


    def write(self):
        self.neoRing.write()
        return


    def brightness(self):
        return


    def prevLedId(self, ledId, n=1):
        return (ledId - n) % self.n        


    def nextLedId(self, ledId, n=1):
        return (ledId + n) % self.n




    def __setitem__(self, index, color):
    
        index = (index + self.indexOffset) % self.n
        self.neoRing[index] = color
        return
        
    def __getitem__(self, index):

        index = (index + self.indexOffset) % self.n
        return(self.neoRing[index])




    
    def __calcBrightness(self, color, brightness):
        r, g, b = color
        r = int(r * brightness)
        g = int(g * brightness)
        b = int(b * brightness)
        return (r, g, b)




    def __calcMixColorAdditive(self, color1, color2):
        r = max(color1[0], color2[0])
        g = max(color1[1], color2[1])
        b = max(color1[2], color2[2])
        return (r, g, b)






if __name__ == "__main__":
    # Erstellen und Starten der Service-Klasse

    import time

    NUM_LEDS = 24
    PIN = 2
    neoRing = NeoPixel(PIN, NUM_LEDS)


    def loop():
        # Display red
        color = (255, 0, 0)  # Red color
        neoRing.fill(color)
        neoRing.write()
        time.sleep(1)

        # Display green
        color = (0, 255, 0)  # Green color
        neoRing.fill(color)
        neoRing.write()
        time.sleep(1)

        # Display blue
        color = (0, 0, 255)  # Blue color
        neoRing.fill(color)
        neoRing.write()
        time.sleep(1)

    while True:
        loop()