import time


import secrets
#### create secrets.py containing:
#### WIFI_SSID = "my_ssid"
#### WIFI_PASSWORD = "my_password"
#### WIFI_COUNTRY = "my_country" e.g. "EN", "DE", ...


from ClockTime import ClockTime
from LedRingClockDisplay import LedRingClockDisplay

import sys
sys.path.append("./lib")
from PicoWlan import PicoWlan
from TimeApiIo import TimeApiIo
from Timer import CyclicTimerMs








class MainClock:

    def __init__(self):
        return

    
    

    def run(self):
    
        clockTime = ClockTime()
        clockDisplay = LedRingClockDisplay()
    
        wlan = PicoWlan(secrets.WIFI_SSID, secrets.WIFI_PASSWORD, secrets.WIFI_COUNTRY)
        wlan.run()
#        time.sleep(2)

        timeApiIo = TimeApiIo()
        timeApiIo.setTimeZone("Europe/Berlin")

        t = timeApiIo.getCurrentTime()
        clockTime.set(t["hour"], t["minute"], t["seconds"])
    
        refreshRate = 50
        refreshPeriodMs = 1000 / refreshRate
        refreshTimer = CyclicTimerMs(refreshPeriodMs)
    
        while True:
            for i in range(refreshRate):
                clockDisplay.displayTime(clockTime.hour, clockTime.minute, clockTime.seconds, i * refreshPeriodMs)
                while not refreshTimer.overrun():
                    time.sleep((0.001 * refreshPeriodMs) / 5)

            clockTime.incrementSeconds()
        # end of while loop






if __name__ == "__main__":

    mainClock = MainClock()
    mainClock.run()


    from mymachine import potentiometer, dipSwitch, buzzer, greyKey, blackKey

    greyKey.setLongTimeMs(3000)


    # To read from the pin, set up a loop:
    while True:
        print(potentiometer.read())

        for i in range(dipSwitch.size()):
            print (">>>>>>>> DIP-Switch: " + str(dipSwitch[i]))
#        buzzer.beep(buzzer.G, buzzer.WHOLE)
        
        event = greyKey.event()
        if event == greyKey.Event.PRESSED:
            print (">>>>>>>> greyKey: PRESSED")
        if event == greyKey.Event.SHORT_PRESS_RELEASED:
            print (">>>>>>>> greyKey: SHORT_PRESS_RELEASED")
        if event == greyKey.Event.LONG_PRESS_DETECTED:
            print (">>>>>>>> greyKey: LONG_PRESS_DETECTED")
        if event == greyKey.Event.LONG_PRESS_RELEASED:
            print (">>>>>>>> greyKey: LONG_PRESS_RELEASED")
        if event == greyKey.Event.RELEASED:
            print (">>>>>>>> greyKey: RELEASED")
        

        time.sleep_ms(1000)
